﻿// Phyllotaxy (formerly "Light Organ") - Jim Bumgardner
//
// Version 2 - added some buttons and sliders

degToRad = Math.PI/180;
SW = Stage.width;
SH = Stage.height;

// build our matrix using the "box" method
doAnimate = true;
doRotate = true;

MovieClip.prototype.drawGradCircle = function(radius, fC)
{
	var alphas = [100,0];
	var ratios = [0x0,0xFF];
	var matrix = { matrixType:"box", x:-radius, y:-radius, w:radius*2, h:radius*2, r: 0 }
	var colors = [fC,fC];

	this.moveTo(-radius,-radius);
  	this.beginGradientFill("radial",colors,alphas,ratios,matrix);
	this.lineTo(radius,-radius);
	this.lineTo(radius,radius);
	this.lineTo(-radius,radius);
	this.lineTo(-radius,-radius);
  	this.endFill()
}

aFreq = .51111;
dFreq = .11101;
gFreq = 2.1111;
bFreq = 2.07;

deg120 = 120*Math.PI/180;
deg240 = 240*Math.PI/180;

var t = 0;

function drawSprite()
{
	if (doAnimate)
		t = getTimer() * .001;

	// Chase is used to determine color
	var chase = this.a * (7+Math.sin(aFreq)*5) + 		// angle (contribution scaled by sin wave)
	            this.d * Math.sin(dFreq)*180*7 + 	// distance from center (ditto)
	            t * 360;																// time

	chase = chase * degToRad;

	var red  = 127+Math.sin(chase)*127;
	var grn  = 127+Math.sin(chase+deg120+Math.sin(t)*gFreq)*127;		// Sin waves used to modify color combinations
	var blu = 127+Math.sin(chase+deg240+Math.cos(t)*bFreq)*127;			// And here...
	var tint = (red << 16) | (grn << 8) | blu;

	// this.clear();

	// drawGradCircle(this,this.r,tint);
	this.itsColor.setRGB(tint);
}

rotSpeed = 1;

function updateFrame()
{
	if (doRotate)
		this._rotation += rotSpeed;
}

var cont = this.createEmptyMovieClip("container", 0);
cont._x = 150;
cont._y = 150;
cont.onEnterFrame = updateFrame;

var m = 256; 		// number of sprites
var rad = SW/2-16;	// radius of outer circle
// var R = 18; 		// max radius of sprites
var phi = (Math.sqrt(5)+1)/2-1; // golden ratio
var ga = phi*2*Math.PI;		// golden angle used in plant phyllotaxis
var asmall = (Math.PI*rad*rad)/m;
var R = Math.sqrt((rad*rad)/m); // also sqrt(asmall/PI);
var kMaxOffset = .9;

var cumA = asmall - asmall*kMaxOffset; // cumulative area
for (var i = 0; i < m; ++i)
{
	var a = i*ga;						// angle of phyllotaxy
	var ir = i/(m-1); // i as a ratio
    var xo = asmall*kMaxOffset*2*ir - asmall*kMaxOffset; // area offset of this nodule
    var areai = asmall+xo;                 // area of this nodule
    var radi = Math.sqrt(areai/Math.PI);             // radius of this nodule
    var radcum = Math.sqrt(cumA/Math.PI);            // radius of cumulative area
	cumA += areai;
	
	var mc = cont.createEmptyMovieClip("sprite_"+i, i);
	mc._x = Math.sin(a) * radcum;
	mc._y = Math.cos(a) * radcum;
	mc.a = a;
	mc.d = radcum/rad;
	mc.n = i;
	mc.r = radi;
	mc.onEnterFrame = drawSprite;
	mc.drawGradCircle(mc.r,0xffffff);
	mc.itsColor = new Color(mc);
}

// UI Stuff
#include "jbumUI.as"

button_time.setupToggleButton("animate", doAnimate, function(t) { doAnimate = t;  },true);
button_rotate.setupToggleButton("rotate", doRotate, function(t) { doRotate = t;  },true);
slider1.setupSlider('twist',0,1, aFreq/(Math.PI*2),function(t) { aFreq = t*Math.PI*2;  bFreq = Math.PI*2-gFreq; } );
slider2.setupSlider('rotation',0,30, rotSpeed,function(t) { rotSpeed = t; } );
slider3.setupSlider('tint',0,1, gFreq/(Math.PI*2),function(t) { gFreq = t*Math.PI*2;  bFreq = Math.PI*2-gFreq; } );


